library IEEE;
library work;

use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;

--! Packages from IP cores
--use work.wishbone_pkg.ALL;
use work.image1_pkg.ALL;
use work.rtm_detector_pkg.ALL;
use work.i2c_slave_pkg.ALL;
use work.m25p32_pkg.ALL;
use work.multiboot_pkg.ALL;

package image1_top_tb_pkg is

   constant c_SCL_I2C_DRIVER_PERIOD  : TIME := 10 us;

   constant c_NUMBER_OF_CHANNELS : NATURAL
                                       := work.image1_pkg.c_NUMBER_OF_CHANNELS;

   component image1_top
   generic(g_NUMBER_OF_CHANNELS : NATURAL := 6);
   port (FPGA_CLK_P        : in STD_LOGIC;
         FPGA_CLK_N        : in STD_LOGIC;

         led_pw_o          : out STD_LOGIC;
         led_err_o         : out STD_LOGIC;
         led_ttl_o         : out STD_LOGIC;
         led_o_front       : out STD_LOGIC_VECTOR(g_NUMBER_OF_CHANNELS downto 1);
         led_o_rear        : out STD_LOGIC_VECTOR(g_NUMBER_OF_CHANNELS downto 1);
         pulse_i_front     : in  STD_LOGIC_VECTOR(g_NUMBER_OF_CHANNELS downto 1);
         pulse_o_front     : out STD_LOGIC_VECTOR(g_NUMBER_OF_CHANNELS downto 1);
         pulse_i_rear      : in  STD_LOGIC_VECTOR(g_NUMBER_OF_CHANNELS downto 1);
         pulse_o_rear      : out STD_LOGIC_VECTOR(g_NUMBER_OF_CHANNELS downto 1);
         inv_i             : in  STD_LOGIC_VECTOR(4 downto 1);
         inv_o             : out STD_LOGIC_VECTOR(4 downto 1);
         --! Lines for the i2c_slave
         SCL_I             : in  STD_LOGIC;
         SCL_O             : out STD_LOGIC;
         SCL_OE            : out STD_LOGIC;
         SDA_I             : in  STD_LOGIC;
         SDA_O             : out STD_LOGIC;
         SDA_OE            : out STD_LOGIC;
         FPGA_GA           : in  STD_LOGIC_VECTOR(4 downto 0);
         FPGA_GAP          : in  STD_LOGIC;
         --! Pins of the SPI interface to write into the Flash memory
         FPGA_PROM_CCLK    : out STD_LOGIC;
         FPGA_PROM_CSO_B_N : out STD_LOGIC;
         FPGA_PROM_DIN     : in  STD_LOGIC;
         FPGA_PROM_MOSI    : out STD_LOGIC;

         --! This LED will show the status of the PLL
         led_link_up_o     : out STD_LOGIC;
         led_pps_o         : out STD_LOGIC;
         led_wr_ok_o       : out STD_LOGIC;
         fpga_o_en         : out STD_LOGIC;
         fpga_o_blo_en     : out STD_LOGIC;
         fpga_o_ttl_en     : out STD_LOGIC;
         fpga_o_inv_en     : out STD_LOGIC;
         level             : in  STD_LOGIC;
         switch_i          : in  STD_LOGIC;   --! General enable
         manual_rst_n_o    : out STD_LOGIC;   --! It allows power sequencing of the
         --! 24V rail after a security given
         --! delay
         FPGA_RTMM         : in STD_LOGIC_VECTOR(2 downto 0);
         FPGA_RTMP         : in STD_LOGIC_VECTOR(2 downto 0));
   end component;

   type t_pulse_vector is
      record
         FRONT : STD_LOGIC_VECTOR(c_NUMBER_OF_CHANNELS - 1 downto 0);
         REAR  : STD_LOGIC_VECTOR(c_NUMBER_OF_CHANNELS - 1 downto 0);
      end record;
   subtype t_pulse_led_vector is t_pulse_vector;

   type t_fpga_en is
      record
         GEN    : STD_LOGIC;
         BLO    : STD_LOGIC;
         TTL    : STD_LOGIC;
         INV    : STD_LOGIC;
      end record;

   type t_I2C_master_out is
      record
         SCL  : STD_LOGIC;
         SDA  : STD_LOGIC;
      end record;
   subtype t_I2C_slave_in is t_I2C_master_out;

   type t_I2C_master_in is
      record
         SDA  : STD_LOGIC;
      end record;
   subtype t_I2C_slave_out is t_I2C_master_in;

   type t_SPI_master_out is
      record
         CCLK    : STD_LOGIC;
         CSO_B_N : STD_LOGIC;
         MOSI    : STD_LOGIC;
      end record;
   subtype t_SPI_slave_in is t_SPI_master_out;

   type t_SPI_master_in is
      record
         DIN     : STD_LOGIC;
      end record;
   subtype t_SPI_slave_out is t_SPI_master_in;

   type t_RTM_id is
      record
         RTMM : STD_LOGIC_VECTOR(2 downto 0);
         RTMP : STD_LOGIC_VECTOR(2 downto 0);
      end record;

   type t_I2C_regs is
      record
         CTR0 : work.i2c_slave_pkg.r_CTR0;
         LT   : work.i2c_slave_pkg.r_LT;
         DRXA : STD_LOGIC_VECTOR(work.i2c_slave_pkg.r_DRX'a_length - 1 downto 0);
         DRXB : STD_LOGIC_VECTOR(work.i2c_slave_pkg.r_DRX'a_length - 1 downto 0);
         DTX  : STD_LOGIC_VECTOR(work.i2c_slave_pkg.r_DTX'a_length - 1 downto 0);
      end record;


   constant c_fpga_en_default  : t_fpga_en := (GEN => '0',
                                               BLO => '0',
                                               TTL => '0',
                                               INV => '0');

   constant c_I2C_master_out_default : t_I2C_master_out := (SCL  => '1',
                                                            SDA  => '1');

   constant c_RTM_id_default : t_RTM_id
                         := (RTMM => STD_LOGIC_VECTOR(c_RTMM_V1),
                             RTMP => STD_LOGIC_VECTOR(c_RTMP_BLOCKING_V1));

   --! ==================================
   --! MEMORY MAPPINGS
   --! ==================================
   --! Byte aligned
   --! It should be noted that all internal modules are word aligned.
   --! Thus, access to byte 1,2 or 3 within an internal word register
   --! will be processed as an access to such register (indeed, like
   --! accessing to byte 0).
   --! ==================================
   --!    MULTIBOOT   [0200-02FF]
   --!    M25P32      [0080-00BF]
   --!    I2C_SLAVE   [0040-007F]
   --! ==================================
   --! c_I2C_CTR0_addr             [0040]
   --! c_I2C_LT_addr               [0044]
   --! c_I2C_DTX_addr              [0048]
   --! c_I2C_DRXA_addr             [004C]
   --! c_I2C_DRXB_addr             [0050]
   --! c_MULTIBOOT_CTR0_addr       [0080]
   --! c_MULTIBOOT_CTR1_addr       [0084]
   --! c_MULTIBOOT_STAT_addr       [0088]
   --! c_MULTIBOOT_MBA_addr        [008C]
   --! c_MULTIBOOT_GBA_addr        [0090]
   --! c_MULTIBOOT_MBA_ICAP_addr   [0094]
   --! c_MULTIBOOT_GBA_ICAP_addr   [0098]
   --! c_M25P32_FMI_addr           [0200]
   --! c_M25P32_SR_m25p32_addr     [0204]
   --! c_M25P32_DATA_READ_addr     [0208]
   --! c_M25P32_DATA_WRITE_addr    [0300 - 03FF]
   --! ==================================

   constant c_I2C_CTR0_addr                    : STD_LOGIC_VECTOR(15 downto 0)
      := X"00"&"01"& STD_LOGIC_VECTOR(work.i2c_slave_pkg.c_CTR0_addr) & "00";
   constant c_I2C_LT_addr                      : STD_LOGIC_VECTOR(15 downto 0)
      := X"00"&"01"& STD_LOGIC_VECTOR(work.i2c_slave_pkg.c_LT_addr)   & "00";
   constant c_I2C_DTX_addr                     : STD_LOGIC_VECTOR(15 downto 0)
      := X"00"&"01"& STD_LOGIC_VECTOR(work.i2c_slave_pkg.c_DTX_addr)  & "00";
   constant c_I2C_DRXA_addr                    : STD_LOGIC_VECTOR(15 downto 0)
      := X"00"&"01"& STD_LOGIC_VECTOR(work.i2c_slave_pkg.c_DRXA_addr) & "00";
   constant c_I2C_DRXB_addr                    : STD_LOGIC_VECTOR(15 downto 0)
      := X"00"&"01"& STD_LOGIC_VECTOR(work.i2c_slave_pkg.c_DRXB_addr) & "00";
   constant c_MULTIBOOT_CTR0_addr              : STD_LOGIC_VECTOR(15 downto 0)
      := X"00"&"10"& STD_LOGIC_VECTOR(work.multiboot_pkg.CTR0_addr) &"00";
   constant c_MULTIBOOT_CTR1_addr              : STD_LOGIC_VECTOR(15 downto 0)
      := X"00"&"10"& STD_LOGIC_VECTOR(work.multiboot_pkg.CTR1_addr) &"00";
   constant c_MULTIBOOT_STAT_addr              : STD_LOGIC_VECTOR(15 downto 0)
      := X"00"&"10"& STD_LOGIC_VECTOR(work.multiboot_pkg.STAT_addr) &"00";
   constant c_MULTIBOOT_MBA_addr               : STD_LOGIC_VECTOR(15 downto 0)
      := X"00"&"10"& STD_LOGIC_VECTOR(work.multiboot_pkg.c_MBA_addr) &"00";
   constant c_MULTIBOOT_GBA_addr               : STD_LOGIC_VECTOR(15 downto 0)
      := X"00"&"10"& STD_LOGIC_VECTOR(work.multiboot_pkg.c_GBA_addr) &"00";
   constant c_MULTIBOOT_MBA_ICAP_addr          : STD_LOGIC_VECTOR(15 downto 0)
      := X"00"&"10"& STD_LOGIC_VECTOR(work.multiboot_pkg.c_MBA_ICAP_addr) &"00";
   constant c_MULTIBOOT_GBA_ICAP_addr          : STD_LOGIC_VECTOR(15 downto 0)
      := X"00"&"10"& STD_LOGIC_VECTOR(work.multiboot_pkg.c_GBA_ICAP_addr) &"00";
   constant c_M25P32_FMI_addr                  : STD_LOGIC_VECTOR(15 downto 0)
      := X"0"&"001"& work.m25p32_pkg.c_FMI_addr        &"00";
   constant c_M25P32_SR_m25p32_addr            : STD_LOGIC_VECTOR(15 downto 0)
      := X"0"&"001"& work.m25p32_pkg.c_SR_m25p32_addr  &"00";
   constant c_M25P32_DATA_READ_addr            : STD_LOGIC_VECTOR(15 downto 0)
      := X"0"&"001"& work.m25p32_pkg.c_DATA_READ_addr  &"00";
   constant c_M25P32_DATA_WRITE_addr           : STD_LOGIC_VECTOR(15 downto 0)
      := X"0"&"001"& work.m25p32_pkg.c_DATA_WRITE_addr &"00";

end image1_top_tb_pkg;

package body image1_top_tb_pkg is
end image1_top_tb_pkg;
